package com.quantechsoftware.darren.deal;

import java.util.ArrayList;
import java.util.List;

import com.quantechsoftware.darren.deal.Deal;
import com.vaadin.flow.component.Composite;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.combobox.ComboBox;
import com.vaadin.flow.component.dialog.Dialog;
import com.vaadin.flow.component.formlayout.FormLayout;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.H2;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.html.Span;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.notification.Notification;
import com.vaadin.flow.component.notification.Notification.Position;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.data.value.ValueChangeMode;

public class DealList extends Composite<VerticalLayout> {
	private Button refresh = new Button("", VaadinIcon.REFRESH.create());
	private Button add = new Button("", VaadinIcon.PLUS.create());
	private Button edit = new Button("", VaadinIcon.PENCIL.create());

	private Grid<Deal> contactGrid = new Grid<>();

	private String dealStatus = "";

	private Dialog contactDialog;
	private Div outerDiv;
	private Button homeButton = new Button();
	private TextField contactCount = new TextField();
	private Button addButton = new Button();

	private List<Deal> getDealList() {
		List<Deal> dealList = new ArrayList<Deal>();

		Deal deal = new Deal();
		deal.setQuid("YUIOYUIDD");
		deal.setDealDate("2018-07-18");
		deal.setDealNumber(1);
		deal.setDealStatus("Pending");
		deal.setDealTotal(5200.00);
		deal.setDealType("Cash");

		if (deal.getDealStatus().equals(dealStatus)) {
			dealList.add(deal);
		}

		Deal deal1 = new Deal();
		deal1.setQuid("YUIUU3DD");
		deal1.setDealDate("2017-07-28");
		deal1.setDealNumber(2);
		deal1.setDealStatus("Inactive");
		deal1.setDealTotal(13200.00);
		deal1.setDealType("Financing");

		if (deal1.getDealStatus().equals(dealStatus)) {
			dealList.add(deal1);
		}

		Deal deal2 = new Deal();
		deal2.setQuid("YUIUTTDD");
		deal2.setDealDate("2018-08-08");
		deal2.setDealNumber(3);
		deal2.setDealStatus("Delivered");
		deal2.setDealTotal(24200.00);
		deal2.setDealType("Financing");

		if (deal2.getDealStatus().equals(dealStatus)) {
			dealList.add(deal2);
		}

		Deal deal3 = new Deal();
		deal3.setQuid("RRIUU3DD");
		deal3.setDealDate("2017-07-28");
		deal3.setDealNumber(4);
		deal3.setDealStatus("Pending");
		deal3.setDealTotal(67500.00);
		deal3.setDealType("Financing");

		if (deal3.getDealStatus().equals(dealStatus)) {
			dealList.add(deal3);
		}

		return dealList;
	}

	public DealList(String sDealStatus) {
		this.dealStatus = sDealStatus;

		initLayout();
		initBehavior();
		refresh();
	}

	private void initLayout() {

		HorizontalLayout header = new HorizontalLayout(refresh, add, edit);

		contactGrid.addColumn(Deal::getDealDate)
				.setComparator((deal1, deal2) -> deal1.getDealDate().compareToIgnoreCase(deal2.getDealDate()))
				.setHeader("Date").setSortable(true);
		contactGrid.addColumn(Deal::getDealNumber)
				.setComparator((deal1, deal2) -> ((Integer) deal1.getDealNumber()).toString()
						.compareToIgnoreCase(((Integer) deal2.getDealNumber()).toString()))
				.setHeader("Deal Number").setSortable(true);
		contactGrid.addColumn(Deal::getDealTotal).setHeader("Total").setSortable(false);
		contactGrid.addColumn(Deal::getDealStatus).setHeader("Status").setSortable(false).setWidth("2%");
		contactGrid.addColumn(Deal::getDealType).setHeader("Type").setSortable(false);
		contactGrid.setColumnReorderingAllowed(true);

		getContent().add(header, contactGrid);
		getContent().expand(contactGrid);
		getContent().setSizeFull();
		getContent().setMargin(false);
		getContent().setPadding(false);

		homeButton.setText("HOME");
		addButton.setText("Add");
	}

	private void initBehavior() {
		contactGrid.asSingleSelect().addValueChangeListener(e -> showEditDialog());
		refresh.addClickListener(e -> refresh());
		add.addClickListener(e -> showAddDialog());
		edit.addClickListener(e -> showEditDialog());

		homeButton.addClickListener(e -> {

			UI.getCurrent().navigate("");

		});

		addButton.addClickListener(e -> {

			UI.getCurrent().navigate("contact-list/");

		});
	}

	public void refresh() {
		contactGrid.setItems(getDealList());
		updateHeader();
	}

	private void deleteClicked(Deal contact) {
		showRemoveDialog(contact);
		refresh();
	}

	private void updateHeader() {
		boolean selected = !contactGrid.asSingleSelect().isEmpty();
		edit.setEnabled(selected);
	}

	private void showAddDialog() {
		DealFormDialog dialog = new DealFormDialog("Add", new Deal());
		dialog.open();
	}

	private void showEditDialog() {
		try {
			DealFormDialog dialog = new DealFormDialog("Edit", contactGrid.asSingleSelect().getValue());
			dialog.open();
		} catch (Exception e) {
			System.out.println("EDIT ERROR? : " + e.getMessage());
		}
	}

	private void showRemoveDialog(Deal contact) {
		RemoveDialog dialog = new RemoveDialog(contact);
		dialog.open();
	}

	private class DealFormDialog extends Dialog {

		private Label label = new Label();
		private TextField dealStatusField = new TextField();
		private TextField dealTotalField = new TextField();
		private TextField dealTypeField = new TextField();

		private Button cancel = new Button("Cancel");
		private Button save = new Button("Save", VaadinIcon.CHECK.create());

		public DealFormDialog(String caption, Deal contact) {
			initLayout(caption, contact);
			initBehavior(contact);
		}

		private void initLayout(String caption, Deal contact) {

			save.getElement().setAttribute("theme", "primary");

			HorizontalLayout buttons = new HorizontalLayout(cancel, save);
			buttons.setSpacing(true);

			label.setText(((Integer) contact.getDealNumber()).toString());
			dealStatusField.setValue(contact.getDealStatus());
			dealTotalField.setValue(((Double) contact.getDealTotal()).toString());
			dealTypeField.setValue(contact.getDealType());

			FormLayout formLayout = new FormLayout(label, dealStatusField, dealTotalField, dealTypeField);

			VerticalLayout layout = new VerticalLayout(formLayout, buttons);
			layout.setAlignSelf(FlexComponent.Alignment.END, buttons);
			add(layout);
		}

		private void initBehavior(Deal deal) {

			cancel.addClickListener(e -> close());
			save.addClickListener(e -> {
				try {

					deal.setDealStatus(dealStatusField.getValue());
					deal.setDealTotal(Double.parseDouble(dealTotalField.getValue()));
					deal.setDealType(dealTypeField.getValue());

					close();

					refresh();

					Notification.show("User saved", 5, Position.TOP_CENTER);

				} catch (Exception ex) {
					Notification.show("Please fix the errors and try again", 5, Position.TOP_CENTER);
				}
			});
		}
	}

	private class RemoveDialog extends Dialog {
		private Button cancel = new Button("Cancel");
		private Button delete = new Button("Delete", VaadinIcon.TRASH.create());

		public RemoveDialog(Deal deal) {
			initLayout(deal);
			initBehavior(deal);
		}

		private void initLayout(Deal deal) {
			Span span = new Span("Do you really want to delete the user " + deal.getDealNumber() + "?");

			delete.getElement().setAttribute("theme", "error");
			HorizontalLayout buttons = new HorizontalLayout(cancel, delete);

			VerticalLayout layout = new VerticalLayout(new H2("Confirm"), span, buttons);
			layout.setAlignSelf(FlexComponent.Alignment.END, buttons);
			add(layout);
		}

		private void initBehavior(Deal contact) {
			cancel.addClickListener(e -> close());

			delete.addClickListener(e -> {
				// UserRepository.delete(contact);
				refresh();
				close();
			});
		}
	}
}
