package com.quantechsoftware.darren.login;

import com.mysql.jdbc.StringUtils;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.server.VaadinService;
import com.vaadin.flow.server.VaadinSession;

import javax.servlet.http.Cookie;

import org.apache.log4j.Logger;

import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * @author Alejandro Duarte
 */
public class AuthService {

	private static Logger oLogger = Logger.getLogger(AuthService.class.getName());

	private static final String USERNAME_ATTRIBUTE = "username";
	private static final String USERQUID_ATTRIBUTE = "user_quid";
	private static final String COOKIE_NAME = "remember-me";
	public static final int COOKIE_MAX_AGE_DAYS = 30;

	private static Map<String, String> remembered = new HashMap<>();
	private static SecureRandom random = new SecureRandom();

	public static boolean authenticate(String username, String password, boolean rememberMe) {
		boolean authentic = "admin".equals(username) && "admin".equals(password);
		String sUserQUID = "HDDED5JW555LH5YJK955";

		if (!StringUtils.isNullOrEmpty(sUserQUID)) {
			if (rememberMe) {
				rememberUser(username);
			}
			VaadinSession.getCurrent().setAttribute(USERNAME_ATTRIBUTE, username);
			VaadinSession.getCurrent().setAttribute(USERQUID_ATTRIBUTE, sUserQUID);
			UI.getCurrent().navigate("");
			UI.getCurrent().getPage().executeJavaScript("location.reload();");

			authentic = true;
		}

		return authentic;
	}

	public static boolean isAuthenticated() {
		if (VaadinSession.getCurrent().getAttribute(USERNAME_ATTRIBUTE) != null) {
			return true;
		} else if (isRememberedUser()) {
			String username = remembered.get(getCookie().get().getValue());
			VaadinSession.getCurrent().setAttribute(USERNAME_ATTRIBUTE, username);
			return true;
		}

		return false;
	}

	public static boolean isRememberedUser() {
		Optional<Cookie> cookie = getCookie();
		return cookie.isPresent() && remembered.containsKey(cookie.get().getValue());
	}

	public static String getAuthenticatedUsername() {
		return (String) VaadinSession.getCurrent().getAttribute(USERNAME_ATTRIBUTE);
	}

	public static String getAuthenticatedUserQUID() {
		return (String) VaadinSession.getCurrent().getAttribute(USERQUID_ATTRIBUTE);
	}

	public static void logout() {
		removeRememberedUser();
		VaadinService.getCurrentRequest().getWrappedSession().invalidate();
		UI.getCurrent().navigate("");
		UI.getCurrent().getPage().executeJavaScript("location.reload();");
	}

	private static void removeRememberedUser() {
		Optional<Cookie> cookie = getCookie();

		if (cookie.isPresent()) {
			remembered.remove(cookie.get().getValue());
			saveCookie("", 0);
		}
	}

	private static void rememberUser(String username) {
		String randomKey = new BigInteger(130, random).toString(32);
		remembered.put(randomKey, username);
		saveCookie(randomKey, 60 * 60 * 24 * COOKIE_MAX_AGE_DAYS);
	}

	private static void saveCookie(String value, int age) {
		Cookie cookie = new Cookie(COOKIE_NAME, value);
		cookie.setPath("/");
		cookie.setMaxAge(age);
		VaadinService.getCurrentResponse().addCookie(cookie);
	}

	private static Optional<Cookie> getCookie() {
		Cookie[] cookies = VaadinService.getCurrentRequest().getCookies();

		if (cookies == null) {
			return Optional.empty();
		}

		return Arrays.stream(cookies).filter(c -> COOKIE_NAME.equals(c.getName())).findFirst();
	}

}
